"""
Take two graphite data files and scatter plot x vs. y.
"""

import datetime
import math
import pytz
import re
import sys

from pylab import *
from scipy import stats

import fetcher

def parse_raw(raw_lines):
    lines = raw_lines.split("\n")

    # Sort by numerical values found in the dataset names (if any)
    def get_nums(s):
        return [int(x) for x in re.findall(r'\d+', s)]

    lines.sort(key=get_nums)

    # print "lines", lines
    # for line in lines:
    #     print "\t", line

    names = []
    start_end_step = None
    data = []
    for raw in [l for l in lines if len(l.strip()) > 0]:

        # print "raw line", raw

        (a, b) = raw.split("|")
        araw = a.split(",")
        name = araw[0]

        def to_float(s):
            try:
                return float(s)
            except:
                return None

        (start, end, step) = [float(s) for s in araw[-3:]]
        data1 = [to_float(s) for s in b.split(",")]
        names.append(name)
        if start_end_step is not None:
            assert start_end_step == (start, end, step)
        else:
            start_end_step = (start, end, step)
        data += data1
    name = ", ".join(names)
    (start, end, step) = start_end_step
    return ((name, start, end, step), data)

def do_plot(xfilespec, yfilespec, label_time):
    print "xfilespec", xfilespec, "yfilespec", yfilespec

    # Returns label, scale, ticks, filename
    def get_params(filespec):
        params = filespec.split(":")
        if len(params) < 4:
            retval = filespec, "1", None, filespec
        else:
            retval = params[0:2] + [int(params[2]), ":".join(params[3:])]
        print "params", retval
        return retval

    x_label, x_scale, x_ticks, xfile = get_params(xfilespec)
    y_label, y_scale, y_ticks, yfile = get_params(yfilespec)

    print "x,y", (x_label, x_scale, x_ticks, xfile), (y_label, y_scale, y_ticks, yfile)

    xraw = open(xfile).read()
    yraw = open(yfile).read()

    # print "xraw", xraw, "yraw", yraw


    xdata = parse_raw(xraw)
    ydata = parse_raw(yraw)

    # print "xdata", repr(xdata), "ydata", repr(ydata)

    print "xrange", xdata[0]
    print "yrange", ydata[0]

    x = array([n / float(eval(x_scale)) if n else 0.0 for n in xdata[1]])
    y = array([n / float(eval(y_scale)) if n else 0.0 for n in ydata[1]])

    print "len(x)", len(x)
    print "len(y)", len(y)

    assert xdata[0][1:] == ydata[0][1:]
    assert len(x) == len(y)

    t1 = xdata[0][1]
    t2 = xdata[0][2]

    # print "t1", t1, "t2", t2

    t1date = datetime.datetime.fromtimestamp(t1, pytz.timezone("UTC"))
    t2date = datetime.datetime.fromtimestamp(t2, pytz.timezone("UTC"))

    drange = "%s to %s" % (t1date, t2date)

    slope, intercept, r_value, p_value, std_err = stats.linregress(x,y)

    print 'r value', r_value
    print  'p_value', p_value
    print 'standard error', std_err
    print '%s = %s * %s + %s' % (y_label, slope, x_label, intercept)

    line = slope*x+intercept
    # plot(x,line,'r-',xi,y,'o')
    # show()

    line_label = "y = %.2g + %.2gx (r=%0.2f)" % (intercept, slope, r_value)
    print "line_label", line_label

    x_min, x_max, y_min, y_max = amin(x), amax(x), amin(y), amax(y)

    # scatter([1,2,3], [2,4,5], s=1, facecolor='0.5', lw = 0)
    plot(x, y, 'k.', alpha=0.1)
    plot(x, line, 'r-')
    axis([x_min, x_max, y_min, y_max])

    def filter_ticks(t, n, min_val, max_val):
        rawloc, lab = t

        print "rawloc", rawloc
        print "labels", lab

        loc = [x for x in rawloc if min_val <= x <= max_val]

        print "filtered loc", loc

        inc = max(len(rawloc) / n, 1)

        print "inc", inc

        print "filtered loc", loc[::inc]

        return loc[::inc]

    if x_ticks:
        xticks(filter_ticks(xticks(), x_ticks, x_min, x_max))
    if y_ticks:
        yticks(filter_ticks(yticks(), y_ticks, y_min, y_max))

    t = "%s vs. %s" % (y_label, x_label)
    if label_time:
        t = "%s\n%s" % (drange, t)
        title(t)

    # if label_time:
    #     title(drange + "\nfilesync activity")

    text(.98 * x_min + .02 * x_max, .93 * y_max + .07 * y_min, line_label,
         color="r", weight="semibold", size="9")
    xlabel(x_label)
    ylabel(y_label)

def main(argv):
    if len(argv) % 2 != 1:
        print "Usage: %s file1 file2 ..." % argv[0]
        print "Or:    %s label:scale:ticks:file1..."
        sys.exit(1)

    pairs = zip(argv[1::2], argv[2::2])

    print "Plotting pairs", pairs

    rows = int(math.sqrt(len(pairs)))
    columns = math.ceil(len(pairs) / float(rows))
    plotnum = 1

    # figure(figsize=(17,8.8))
    figure(figsize=(17*.8,8.8*.8))

    label_time = True
    for i, j in pairs:

        print "Adding plot %s on %s x %s grid" % (plotnum, rows, columns)

        subplot(rows, columns, plotnum)
        do_plot(i, j, label_time)
        plotnum += 1
        label_time = False

    subplots_adjust(hspace=.35, wspace=.35)

if __name__ == '__main__':
    main(sys.argv)
    show()

